import tkinter as tk
import tkinter.ttk as ttk

from ...config import config, Mode

from ...backend.tk import widgets
from ...backend.mpl import theme as theme_mpl

from .panels import PanelView

from .base import View as _View

class ViewFile(_View):
    __slots__ = (
        # toolbar
        "btn_prev", "btn_next",
        "var_samp", "sld_samp",
        "var_time_cur", "lbl_time_cur",
        "var_time_tot", "lbl_time_tot",
        # control panel
        "var_file", "btn_file", "ent_file",
        "var_file_fmt", "cb_file_fmt"
    )

    def __init__(self, *args, **kwargs):
        # toolbar
        self.btn_prev: ttk.Button = None # type: ignore
        self.btn_next: ttk.Button = None # type: ignore
        self.var_samp: tk.IntVar = None # type: ignore
        self.sld_samp: widgets.Scale = None # type: ignore
        self.var_time_cur: tk.StringVar = None # type: ignore
        self.lbl_time_cur: ttk.Label = None # type: ignore
        self.var_time_tot: tk.StringVar = None # type: ignore
        self.lbl_time_tot: ttk.Label = None # type: ignore
        # control panel
        self.var_file: tk.StringVar = None # type: ignore
        self.btn_file: ttk.Button = None # type: ignore
        self.ent_file: ttk.Entry = None # type: ignore
        self.var_file_fmt: tk.StringVar = None # type: ignore
        self.cb_file_fmt: ttk.Combobox = None # type: ignore
        super().__init__(*args, **kwargs)

    def draw_tb_progress(self, parent, col=0):
        self.var_samp = tk.IntVar(parent)
        self.sld_samp = widgets.Scale(
            parent, variable=self.var_samp, length=150
        )
        self.sld_samp.grid(row=0,rowspan=2,column=col, sticky=tk.NSEW)
        col += 1
        self.var_time_cur = tk.StringVar(parent)
        self.var_time_tot = tk.StringVar(parent)
        self.lbl_time_cur = ttk.Label(parent, textvariable=self.var_time_cur)
        self.lbl_time_cur.grid(row=0,column=col)
        self.lbl_time_tot = ttk.Label(parent, textvariable=self.var_time_tot)
        self.lbl_time_tot.grid(row=1,column=col)
        col += 1
        ttk.Separator(parent, orient=tk.VERTICAL).grid(row=0,rowspan=2,column=col, padx=5, sticky=tk.NS)
        col += 1
        return col

    def draw_tb_btn(self, parent, col=0):
        self.btn_prev = ttk.Button(parent, text="Prev")
        self.btn_prev.grid(row=0,rowspan=2,column=col, padx=2,pady=2)
        col += 1
        self.btn_next = ttk.Button(parent, text="Next")
        self.btn_next.grid(row=0,rowspan=2,column=col, padx=2,pady=2)
        col += 1
        return super().draw_tb_btn(parent, col)

    def draw_ctrl_sink(self, parent):
        root = ttk.Frame(parent) # Sink.FILE
        root.columnconfigure(2, weight=1)
        row = 0
        self.var_file = tk.StringVar(root)
        self.btn_file = ttk.Button(root, text="File")
        self.btn_file.grid(row=row,column=0, sticky=tk.W)
        self.ent_file = ttk.Entry(root, textvariable=self.var_file, state=tk.DISABLED, width=10)
        self.ent_file.grid(row=row,column=1,columnspan=2, sticky=tk.NSEW)
        row += 1
        ttk.Label(root, text="Format:").grid(row=row,column=0,sticky=tk.W)
        self.var_file_fmt = tk.StringVar(root)
        self.cb_file_fmt = ttk.Combobox(root, textvariable=self.var_file_fmt, width=5)
        self.cb_file_fmt.grid(row=row,column=1, sticky=tk.W)
        root.pack(padx=2,pady=2, fill=tk.X)



"""Create a tkGUI view"""

from ..config import Mode, Sink
from .tkGUI.base import View
from .tkGUI.sink_file import ViewFile
from .tkGUI.sink_live import ViewLive

def GetView(mode, sink):
    if sink == Sink.FILE:
        return ViewFile
    elif sink == Sink.LIVE:
        return ViewLive
    return View



import tkinter as tk
import tkinter.ttk as ttk

from ...config import config, Mode

from ...backend.tk import widgets
from ...backend.mpl import theme as theme_mpl

from .panels import PanelView

from .base import View as _View

class ViewLive(_View):
    __slots__ = (
        # control panel
        "var_dev", "cb_dev",
        "fr_rx_rf", "var_rx_rf", "ent_rx_rf",
        "fr_rx_if", "var_rx_if", "ent_rx_if",
        "fr_rx_bb", "var_rx_bb", "ent_rx_bb"
    )

    def __init__(self, *args, **kwargs):
        self.var_dev: tk.StringVar = None # type: ignore
        self.cb_dev: ttk.Combobox = None # type: ignore
        self.fr_rx_rf: ttk.Frame = None # type: ignore
        self.var_rx_rf: tk.StringVar = None # type: ignore
        self.ent_rx_rf: ttk.Entry = None # type: ignore
        self.fr_rx_if: ttk.Frame = None # type: ignore
        self.var_rx_if: tk.StringVar = None # type: ignore
        self.ent_rx_if: ttk.Entry = None # type: ignore
        self.fr_rx_bb: ttk.Frame = None # type: ignore
        self.var_rx_bb: tk.StringVar = None # type: ignore
        self.ent_rx_bb: ttk.Entry = None # type: ignore
        super().__init__(*args, **kwargs)
    def draw_tb_progress(self, parent, col=0):
        return col

    def draw_ctrl_sink(self, parent):
        root = ttk.Frame(parent) # Sink.LIVE
        root.columnconfigure(1, weight=1)
        row = 0
        self.var_dev = tk.StringVar(root) # TODO: switch from ent to combo
        ttk.Label(root, text="Device:").grid(row=row,column=0, sticky=tk.W)
        self.cb_dev = ttk.Combobox(root, textvariable=self.var_dev, width=20)
        # self.ent_dev = ttk.Entry(root, textvariable=self.var_dev, state=tk.DISABLED, width=10)
        self.cb_dev.grid(row=row,column=1, sticky=tk.NSEW)
        row += 1
        ttk.Separator(root, orient=tk.HORIZONTAL).grid(row=row, column=0, columnspan=2, pady=5, sticky=tk.EW)
        row += 1
        self.fr_rx_rf = ttk.Frame(root)
        self.fr_rx_rf.columnconfigure(1, weight=1)
        self.var_rx_rf = tk.StringVar(self.fr_rx_rf)
        ttk.Label(self.fr_rx_rf, text="Rx RF Gain:").grid(row=0,column=0, sticky=tk.W)
        self.ent_rx_rf = ttk.Entry(self.fr_rx_rf, textvariable=self.var_rx_rf, width=3)
        self.ent_rx_rf.grid(row=0,column=1, sticky=tk.E)
        self.fr_rx_rf.grid(row=row,column=0,columnspan=2, sticky=tk.NSEW)
        # root.columnconfigure(row, weight=1)
        row += 1
        self.fr_rx_if = ttk.Frame(root)
        self.fr_rx_if.columnconfigure(1, weight=1)
        self.var_rx_if = tk.StringVar(self.fr_rx_if)
        ttk.Label(self.fr_rx_if, text="Rx IF Gain:").grid(row=0,column=0, sticky=tk.W)
        self.ent_rx_if = ttk.Entry(self.fr_rx_if, textvariable=self.var_rx_if, width=3)
        self.ent_rx_if.grid(row=0,column=1, sticky=tk.E)
        self.fr_rx_if.grid(row=row,column=0,columnspan=2, sticky=tk.NSEW)
        # root.columnconfigure(row, weight=1)
        row += 1
        self.fr_rx_bb = ttk.Frame(root)
        self.fr_rx_bb.columnconfigure(1, weight=1)
        self.var_rx_bb = tk.StringVar(self.fr_rx_bb)
        ttk.Label(self.fr_rx_bb, text="Rx BB Gain:").grid(row=0,column=0, sticky=tk.W)
        self.ent_rx_bb = ttk.Entry(self.fr_rx_bb, textvariable=self.var_rx_bb, width=3)
        self.ent_rx_bb.grid(row=0,column=1, sticky=tk.E)
        self.fr_rx_bb.grid(row=row,column=0,columnspan=2, sticky=tk.NSEW)
        # root.columnconfigure(row, weight=1)
        root.pack(padx=2,pady=2, fill=tk.X)



import argparse
from ...utils import args

from .mode import Mode, args_mode
from ...utils import stft

class ModeConfig:
    overlap = 0.6
    block_max = 102400

def args_rt(parser: argparse.ArgumentParser):
    mode = args.get_group(parser, "Mode (RT)")
    args_mode(mode)
    mode.add_argument("--overlap", default=ModeConfig.overlap, type=float)
    mode.add_argument("--block_max", default=ModeConfig.block_max, type=int)

class ModeRT(Mode):
    __slots__ = (
        "_overlap", "_block_max"
    )
    def __init__(self, model, **kwargs):
        super().__init__(model)
        self._overlap = kwargs.get("overlap", ModeConfig.overlap)
        self._block_max = kwargs.get("block_max", ModeConfig.block_max)
        self.update_blocksize()

    def update_blocksize(self):
        self._block_size = int(self.model.get_fs() * (self._sweep_time/1000))
        if self._block_size > self._block_max:
            self._block_size = self._block_max
            super().set_sweep_time((self._block_size/self.model.get_fs())*1000)

    def get_overlap(self):
        return self._overlap
    def set_overlap(self, overlap):
        if overlap <= 0.0 or overlap > 1.0:
            raise ValueError
        self._overlap = float(overlap)
    overlap = property(get_overlap, set_overlap)

    def set_sweep_time(self, ts):
        super().set_sweep_time(ts)
        self.update_blocksize()



"""Create a GUI view"""
import tkinter as tk
import tkinter.ttk as ttk

from ..base import View as _View
from ...config import config, Mode

from ...backend.tk import widgets
from ...backend.mpl import theme as theme_mpl

from .panels import PanelView

class View(_View):
    """Parent tkGUI view class"""

    __slots__ = (
        "root", "_main",
        "main", "fr_tb", "fr_view", "fr_ctrl", "frame",
        # toolbar
        "btn_start", "btn_stop", "btn_reset",
        "var_sweep", "ent_sweep",
        "var_show", "ent_show",
        "lbl_msg"
        "var_draw_time", "lbl_draw_time",
        # control panel
        "var_fs", "ent_fs",
        "var_cf", "ent_cf",
        "var_nfft_exp", "ent_nfft_exp",
        "lbl_nfft",
        "lbl_block_size", "lbl_sweep_samples",
    )
    def __init__(self, root=tk.Tk(), **kwargs):
        self.root = root
        self.root.title("pyspecan")

        if config.MODE == Mode.SWEPT:
            self.root.title("pyspecan | Swept")
        elif config.MODE == Mode.RT:
            self.root.title("pyspecan | Real-Time")

        theme_mpl.get(kwargs.get("theme", "Dark"))() # Set matplotlib theme

        # self.style = ttk.Style(root)

        self._main = ttk.Frame(self.root)
        self._main.pack(expand=True, fill=tk.BOTH)

        self.fr_tb = ttk.Frame(self._main, height=20)
        self.btn_start: ttk.Button = None # type: ignore
        self.btn_stop: ttk.Button = None # type: ignore
        self.btn_reset: ttk.Button = None # type: ignore
        self.var_sweep: tk.StringVar = None # type: ignore
        self.ent_sweep: ttk.Entry = None # type: ignore
        self.var_show: tk.StringVar = None # type: ignore
        self.ent_show: ttk.Entry = None # type: ignore
        self.lbl_msg: ttk.Label = None # type: ignore
        self.var_draw_time: tk.StringVar = None # type: ignore
        self.lbl_draw_time: ttk.Label = None # type: ignore
        self.draw_tb(self.fr_tb)
        self.fr_tb.pack(side=tk.TOP, fill=tk.X)

        self.main = ttk.PanedWindow(self._main, orient=tk.HORIZONTAL)
        self.main.pack(side=tk.TOP, fill=tk.BOTH, expand=True)

        self.fr_view = ttk.Frame(self.main)
        self.panel = PanelView(self.fr_view)

        self.fr_ctrl = ttk.Frame(self.main, width=100)
        self.var_fs: tk.StringVar = None # type: ignore
        self.ent_fs: ttk.Entry = None # type: ignore
        self.var_cf: tk.StringVar = None # type: ignore
        self.ent_cf: ttk.Entry = None # type: ignore
        self.var_nfft_exp: tk.StringVar = None # type: ignore
        self.ent_nfft_exp: ttk.Entry = None # type: ignore
        self.lbl_nfft: ttk.Label = None # type: ignore
        self.lbl_block_size: ttk.Label = None # type: ignore
        self.lbl_sweep_samples: ttk.Label = None # type: ignore
        self.draw_ctrl(self.fr_ctrl)
        self.main.add(self.fr_ctrl)

        self.main.add(self.fr_view)

    def draw_tb(self, parent, col=0):
        """Draw toolbar frame"""
        col = self.draw_tb_progress(parent, col)
        col = self.draw_tb_time(parent, col)
        col = self.draw_tb_btn(parent, col)
        col = self.draw_tb_msg(parent, col)

    def draw_tb_time(self, parent, col=0):
        ttk.Label(parent, text="Sweep").grid(row=0,column=col)
        self.var_sweep = tk.StringVar(parent)
        self.ent_sweep = ttk.Entry(parent, textvariable=self.var_sweep, width=8)
        self.ent_sweep.grid(row=1,column=col, padx=2, pady=2)
        col += 1
        ttk.Label(parent, text="Show").grid(row=0,column=col)
        self.var_show = tk.StringVar(parent)
        self.ent_show = ttk.Entry(parent, textvariable=self.var_show, width=8)
        self.ent_show.grid(row=1,column=col, padx=2, pady=2)
        col += 1
        ttk.Separator(parent, orient=tk.VERTICAL).grid(row=0,rowspan=2,column=col, padx=5, sticky=tk.NS)
        col += 1
        return col

    def draw_tb_btn(self, parent, col=0):
        self.btn_start = ttk.Button(parent, text="Start")
        self.btn_start.grid(row=0,rowspan=2,column=col, padx=2,pady=2, sticky=tk.NS)
        col += 1
        self.btn_stop = ttk.Button(parent, text="Stop", state=tk.DISABLED)
        self.btn_stop.grid(row=0,rowspan=2,column=col, padx=2,pady=2, sticky=tk.NS)
        col += 1
        self.btn_reset = ttk.Button(parent, text="Reset")
        self.btn_reset.grid(row=0,rowspan=2,column=col, padx=2,pady=2, sticky=tk.NS)
        col += 1
        ttk.Separator(parent, orient=tk.VERTICAL).grid(row=0,rowspan=2,column=col, padx=5, sticky=tk.NS)
        col += 1
        return col

    def draw_tb_msg(self, parent, col=0):
        self.lbl_msg = ttk.Label(parent, text="")
        self.lbl_msg.grid(row=0,column=col, rowspan=2, sticky=tk.E)
        parent.grid_columnconfigure(col, weight=1)
        col += 1
        self.var_draw_time = tk.StringVar(parent)
        self.lbl_draw_time = ttk.Label(parent, textvariable=self.var_draw_time)
        ttk.Label(parent, text="Draw").grid(row=0,column=col, sticky=tk.E)
        self.lbl_draw_time.grid(row=1,column=col, sticky=tk.E)
        parent.grid_columnconfigure(col, weight=1)
        return col

    def draw_ctrl(self, parent):
        """Draw control frame"""
        self.draw_ctrl_sink(parent)
        self.draw_ctrl_view(parent)
        self.draw_ctrl_lbls(parent)

    def draw_ctrl_view(self, parent):
        root = ttk.Frame(parent) # View settings
        root.columnconfigure(2, weight=1)
        row = 0
        self.var_fs = tk.StringVar(root)
        ttk.Label(root, text="Sample rate:").grid(row=row,column=0, sticky=tk.W)
        self.ent_fs = ttk.Entry(root, textvariable=self.var_fs, width=10)
        self.ent_fs.grid(row=row,column=1, columnspan=2,sticky=tk.E)
        row += 1
        self.var_cf = tk.StringVar(root)
        ttk.Label(root, text="Center freq:").grid(row=row,column=0, sticky=tk.W)
        self.ent_cf = ttk.Entry(root, textvariable=self.var_cf, width=10)
        self.ent_cf.grid(row=row,column=1, columnspan=2,sticky=tk.E)
        row += 1
        self.var_nfft_exp = tk.StringVar(root)
        ttk.Label(root, text="NFFT 2^").grid(row=row,column=0, sticky=tk.W)
        self.ent_nfft_exp = ttk.Entry(root, textvariable=self.var_nfft_exp, width=2)
        self.ent_nfft_exp.grid(row=row,column=1, sticky=tk.W)
        self.lbl_nfft = ttk.Label(root)
        self.lbl_nfft.grid(row=row,column=2, sticky=tk.E)
        root.pack(padx=2,pady=2, fill=tk.X)

    def draw_ctrl_lbls(self, parent):
        ttk.Separator(parent, orient=tk.HORIZONTAL).pack(fill=tk.X, pady=5)

        root = ttk.Frame(parent) # Useful variables
        row = 0
        ttk.Label(root, text="Block size: ").grid(row=row, column=0, sticky=tk.W)
        self.lbl_block_size = ttk.Label(root)
        self.lbl_block_size.grid(row=row, column=1)
        row += 1
        ttk.Label(root, text="Sweep size: ").grid(row=row, column=0, sticky=tk.W)
        self.lbl_sweep_samples = ttk.Label(root)
        self.lbl_sweep_samples.grid(row=row, column=1)
        root.pack(padx=2,pady=2, fill=tk.X)

    def mainloop(self):
        self.root.mainloop()

    # --- Defined in child classes --- #
    def draw_tb_progress(self, parent, col=0):
        raise NotImplementedError()
    def draw_ctrl_sink(self, parent):
        raise NotImplementedError()
